//
//	plasticfs - the plastic file system
//	Copyright (C) 2002 Peter Miller;
//	All rights reserved.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
//
// MANIFEST: interface definition for lib/errno_insulator.cc
//

#ifndef LIB_ERRNO_INSULATOR_H
#define LIB_ERRNO_INSULATOR_H

#pragma interface "libplasticfs::errno_insulator"

namespace libplasticfs {

/**
  * The errno_insulator class is used to preserve the value of errno
  * across a symbol scope.  It only exists for its constructor and
  * destructor; no matter how the scope is left (break, return, exception,
  * etc) the value of errno will *always* be restored.
  */
class errno_insulator
{
public:
    /**
      * The destructor.
      */
    ~errno_insulator();

    /**
      * The default constructor.
      */
    errno_insulator();

    /**
      * The get method is used to get the value of errno being held.
      */
    int get() const { return hold; }

    /**
      * The gets method is used to get the human readable equivalent of
      * the errno value being held.
      */
    const char *gets() const;

private:
    /**
      * The hold instance variable is used to hold the value of errno
      * across the scope.
      */
    int hold;

    /**
      * The copy constructor.  Do not use.
      */
    errno_insulator(const errno_insulator &);

    /**
      * The assignment operator.  Do not use.
      */
    errno_insulator &operator=(const errno_insulator &);
};

} // namespace libplasticfs

#endif // LIB_ERRNO_INSULATOR_H
