//
//	plasticfs - the plastic file system
//	Copyright (C) 2003, 2004 Peter Miller;
//	All rights reserved.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
//
// MANIFEST: functions to impliment the fcntl class
//

#include <plasticfs.h>
#include <cstdarg>
#include <fcntl.h>

using namespace libplasticfs;


extern "C" int
fcntl(int filedesc, int cmd, ...)
{
    //
    // Get the argument value, very carefully.
    //
    void *arg = 0;
    va_list ap;
    va_start(ap, cmd);
    switch (cmd)
    {
    case F_DUPFD:
    case F_SETFD:
    case F_SETFL:
    case F_SETOWN:
    case F_SETSIG:
	arg = (void *)va_arg(ap, long);
	break;

    case F_GETLK:
    case F_SETLK:
    case F_SETLKW:
	arg = (void *)va_arg(ap, flock *);
	break;

    case F_GETLK64:
    case F_SETLK64:
    case F_SETLKW64:
	//
        // This is needed for RPM to work, apparently, but I don't
        // understand why strace shows RPM as calling fcntl64 but it
        // still ends up here!
	//
	arg = (void *)va_arg(ap, flock64 *);
	break;

    default:
	//
        // Catch-all default.  It will work for all machines where
        // pointers and longs and ints are all the same size.
	// GNU Libc often does this.
	//
	arg = va_arg(ap, void *);
	break;
    }
    va_end(ap);

    //
    // Now call the method, it always has 3 arguments.
    //
    return plasticfs::get_master()->fcntl(filedesc, cmd, arg);
}


extern "C" int
__fcntl(int filedesc, int cmd, ...)
{
    //
    // Get the argument value, very carefully.
    //
    void *arg = 0;
    va_list ap;
    va_start(ap, cmd);
    switch (cmd)
    {
    case F_DUPFD:
    case F_SETFD:
    case F_SETFL:
    case F_SETOWN:
    case F_SETSIG:
	arg = (void *)va_arg(ap, long);
	break;

    case F_GETLK:
    case F_SETLK:
    case F_SETLKW:
	arg = (void *)va_arg(ap, flock *);
	break;

    case F_GETLK64:
    case F_SETLK64:
    case F_SETLKW64:
	arg = (void *)va_arg(ap, flock64 *);
	break;

    default:
	//
        // Catch-all default.  It will work for all machines where
        // pointers and longs and ints are all the same size.
	// GNU Libc often does this.
	//
	arg = va_arg(ap, void *);
	break;
    }
    va_end(ap);

    //
    // Now call the method, it always has 3 arguments.
    //
    int result = plasticfs::get_master()->fcntl(filedesc, cmd, arg);
    return result;
}


extern "C" int
fcntl64(int filedesc, int cmd, ...)
{
    //
    // Get the argument value, very carefully.
    //
    void *arg = 0;
    va_list ap;
    va_start(ap, cmd);
#if 1
    // This is how glibc does it, even though it looks wrong. 
    arg = va_arg(ap, void *);
#else
    // You may think that this is how it needs to be done,
    // but glibc does it differently.
    switch (cmd)
    {
    case F_DUPFD:
    case F_SETFD:
    case F_SETFL:
    case F_SETOWN:
    case F_SETSIG:
	arg = (void *)va_arg(ap, long);
	break;

    case F_GETLK:
    case F_SETLK:
    case F_SETLKW:
	arg = (void *)va_arg(ap, flock *);
	break;

    default:
	//
        // Catch-all default.  It will work for all machines where
        // pointers and longs and ints are all the same size.
	// GNU Libc often does this.
	//
	arg = va_arg(ap, void *);
	break;
    }
#endif
    va_end(ap);

    //
    // Now call the method, it always has 3 arguments.
    //
    return plasticfs::get_master()->fcntl64(filedesc, cmd, arg);
}


extern "C" int
__fcntl64(int filedesc, int cmd, ...)
{
    //
    // Get the argument value, very carefully.
    //
    void *arg = 0;
    va_list ap;
    va_start(ap, cmd);
#if 1
    // This is how glibc does it, even though it looks wrong. 
    arg = va_arg(ap, void *);
#else
    // You may think that this is how it needs to be done,
    // but glibc does it differently.
    switch (cmd)
    {
    case F_DUPFD:
    case F_SETFD:
    case F_SETFL:
    case F_SETOWN:
    case F_SETSIG:
	arg = (void *)va_arg(ap, long);
	break;

    case F_GETLK:
    case F_SETLK:
    case F_SETLKW:
	arg = (void *)va_arg(ap, flock *);
	break;

    case F_GETLK64:
    case F_SETLK64:
    case F_SETLKW64:
	arg = (void *)va_arg(ap, flock64 *);
	break;

    default:
	//
        // Catch-all default.  It will work for all machines where
        // pointers and longs and ints are all the same size.
	// GNU Libc often does this.
	//
	arg = va_arg(ap, void *);
	break;
    }
#endif
    va_end(ap);

    //
    // Now call the method, it always has 3 arguments.
    //
    return plasticfs::get_master()->fcntl64(filedesc, cmd, arg);
}
