//
//	plasticfs - the plastic file system
//	Copyright (C) 2003 Peter Miller;
//	All rights reserved.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
//
// MANIFEST: functions to impliment the file_tracker class
//

#pragma implementation "libplasticfs::file_tracker"

#include <file_tracker.h>
#include <errno_insulator.h>
#include <plasticfs.h>

using namespace libplasticfs;


static void
reaper(void *p)
{
    file_tracker::active_t *ap = (file_tracker::active_t *)p;
    ap->close();
}


file_tracker::~file_tracker()
{
}


file_tracker::file_tracker()
{
    active.set_reaper(reaper);
}


file_tracker::active_t *
file_tracker::open(int fd, int flags, const rcstring &relpath, plasticfs *fs)
{
    active_t *p = new active_t;
    if (!p)
	return 0;
    p->flags = flags;
    p->relpath = relpath;
    p->fs = fs;
    active.assign(fd, p);
    return p;
}


void
file_tracker::close(int fd)
{
    active_t *p = (active_t *)active.query(fd);
    if (p)
    {
	active.remove(fd);
	p->close();
    }
}


void
file_tracker::active_t::close()
{
    //
    // Need to guard against memory scribbles in the application
    // trashing our data structure.
    //
    if (magic != MAGIC)
	return;

    reference_count--;
    if (reference_count > 0)
	return;

    magic = 0;
    if (sub_fd >= 0)
    {
	errno_insulator guard;
	fs->close(sub_fd);
    }
    delete this;
}


void
file_tracker::dup(int old_fd, int new_fd)
{
    active_t *p = find(old_fd);
    if (p)
    {
	p->reference_count++;
	active.assign(new_fd, p);
    }
}
