//
//	plasticfs - the plastic file system
//	Copyright (C) 2003 Peter Miller;
//	All rights reserved.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
//
// MANIFEST: functions to impliment the getdents class
//

#include <plasticfs.h>

using namespace libplasticfs;

//#define DEBUG

#ifdef HAVE_GETDENTS
//
// If libc.so does not export getdents, we must not either, otherwise
// libc.so's internal use of getdents will resolve to us, causing an
// infinite recursion.
//


extern "C" int
getdents(int fd, dirent *buf, unsigned count)
{
    #ifdef DEBUG
    plasticfs::error_message("%s: %d: getdents(fd = %d, buffer = %p, "
	"nbytes = %u)\n", __FILE__, __LINE__, fd, buffer, nbytes);
    #endif

    return plasticfs::get_master()->getdents(fd, buf, count);
}


extern "C" int
__getdents(int fd, dirent *buf, unsigned count)
{
    #ifdef DEBUG
    plasticfs::error_message("%s: %d: __getdents(fd = %d, buffer = %p, "
	"nbytes = %lu)\n", __FILE__, __LINE__, fd, buffer, nbytes);
    #endif

    return plasticfs::get_master()->getdents(fd, buf, count);
}


extern "C" int
getdents64(int fd, dirent64 *buf, unsigned count)
{
    #ifdef DEBUG
    plasticfs::error_message("%s: %d: getdents64(fd = %d, buffer = %p, "
	"nbytes = %u)\n", __FILE__, __LINE__, fd, buffer, nbytes);
    #endif

    return plasticfs::get_master()->getdents64(fd, buf, count);
}


extern "C" int
__getdents64(int fd, dirent64 *buf, unsigned count)
{
    #ifdef DEBUG
    plasticfs::error_message("%s: %d: __getdents64(fd = %d, buffer = %p, "
	"nbytes = %u)\n", __FILE__, __LINE__, fd, buffer, nbytes);
    #endif

    return plasticfs::get_master()->getdents64(fd, buf, count);
}

#endif
