//
//	plasticfs - the plastic file system
//	Copyright (C) 2003 Peter Miller;
//	All rights reserved.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
//
// MANIFEST: functions to impliment the input class
//

#pragma implementation "libplasticfs::input"

#include <input.h>
#include <rcstring.h>

using namespace libplasticfs;


input::~input()
{
}


input::input() :
    bufmax(buffer),
    bufpos(buffer)
{
}


bool
input::readline(rcstring &result)
{
    // I should probably allow arbitrarily long lines,
    // but I won't bother until it's a problem.
    // The immediate use is file name entries, limited to 255 characters.
    char buffer[2000];
    char *end = buffer + sizeof(buffer);
    char *bp = buffer;
    for (;;)
    {
	int c = getc();
	if (c == EOF)
	{
	    if (bp == buffer)
		return false;
	    result = rcstring(buffer, bp - buffer);
	    return true;
	}
	if (c == '\n')
	{
	    result = rcstring(buffer, bp - buffer);
	    return true;
	}
	if (bp < end)
	    *bp++ = c;
    }
}


bool
input::readline_c(rcstring &result)
{
    // I should probably allow arbitrarily long lines,
    // but I won't bother until it's a problem.
    char buffer[2000];
    char *end = buffer + sizeof(buffer);
    char *bp = buffer;
    for (;;)
    {
	int c = getc();
	if (c == EOF)
	{
	    if (bp == buffer)
		return false;
	    result = rcstring(buffer, bp - buffer);
	    return true;
	}
	if (c == '\n')
	{
	    result = rcstring(buffer, bp - buffer);
	    return true;
	}
	if (c == '\\')
	{
	    c = getc();
	    switch (c)
	    {
	    default:
		// pass through escapes we don't understand.
		//
		// No C octal escape.  Add them if needed
		// (also, they require ungetc, which is a pain).
		break;

	    case EOF:
		c = '\\';
		break;

	    case '\n':
		continue;

	    case 'a': c = '\a'; break;
	    case 'b': c = '\b'; break;
	    case 'f': c = '\f'; break;
	    case 'n': c = '\n'; break;
	    case 'r': c = '\r'; break;
	    case 't': c = '\t'; break;
	    case 'v': c = '\v'; break;
	    }
	}
	if (bp < end)
	    *bp++ = c;
    }
}
