//
//	plasticfs - the plastic file system
//	Copyright (C) 2003 Peter Miller;
//	All rights reserved.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
//
// MANIFEST: interface definition for lib/input/file.cc
//

#ifndef LIB_INPUT_FILE_H
#define LIB_INPUT_FILE_H

#pragma interface "libplasticfs::input_file"

#include <input.h>

#include <plasticfs.h>

namespace libplasticfs {

/**
  * The input_file class is used to represent an input stream originating
  * from a file.
  */
class input_file:
    public input
{
public:
    /**
      * The destructor.
      */
    virtual ~input_file();

    /**
      * The constructor.
      */
    input_file(plasticfs *filesys, const rcstring &filename);

    // See base class for documentation.
    ssize_t underflow(void *, size_t);

private:
    /**
      * The fs instance variable is used to remember which file system
      * to use to read the file from.
      */
    plasticfs *fs;

    /**
      * The fn instance variable is used to remember which file to read
      * the data from.
      */
    rcstring fn;

    /**
      * The fd instance variable is used to remember the file descriptor
      * when the file is open.  It is -1 when the file is closed.
      */
    int fd;

    /**
      * The default constructor.  Do not use.
      */
    input_file();

    /**
      * The copy constructor.  Do not use.
      */
    input_file(const input_file &);

    /**
      * The assignment operator.  Do not use.
      */
    input_file &operator=(const input_file &);
};

} // namespace libplasticfs

#endif // LIB_INPUT_FILE_H
