//
//	plasticfs - the plastic file system
//	Copyright (C) 2002, 2003 Peter Miller;
//	All rights reserved.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
//
// MANIFEST: functions to impliment the open64 class
//

#define _LARGEFILE_SOURCE
#define _LARGEFILE64_SOURCE

#include <stdarg.h>
#include <fcntl.h>
#include <plasticfs.h>
#include <cerrno>

using namespace libplasticfs;


extern "C" int
open64(const char *path, int flags, ...)
{
    va_list ap;
    va_start(ap, flags);
    mode_t mode = 0;
    if (flags & O_CREAT)
	mode = va_arg(ap, mode_t);
    va_end(ap);

    if (!path)
    {
	errno = EINVAL;
	return -1;
    }

    /*
     * This is exactly what glibc's sysdeps/posix/open64.c does,
     * although it goes direct to __libc_open and we redirect through open.
     */
    return plasticfs::get_master()->open(path, flags | O_LARGEFILE, mode);
}


extern "C" int
__open64(const char *path, int flags, ...)
{
    va_list ap;
    va_start(ap, flags);
    mode_t mode = 0;
    if (flags & O_CREAT)
	mode = va_arg(ap, mode_t);
    va_end(ap);

    if (!path)
    {
	errno = EINVAL;
	return -1;
    }

    /*
     * This is exactly what glibc's sysdeps/posix/open64.c does,
     * although it goes direct to __libc_open and we redirect through __open.
     */
    return plasticfs::get_master()->open(path, flags | O_LARGEFILE, mode);
}
