//
//	plasticfs - the plastic file system
//	Copyright (C) 2003 Peter Miller;
//	All rights reserved.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
//
// MANIFEST: functions to impliment the output_file class
//

#pragma implementation "libplasticfs::output_file"

#include <output/file.h>
#include <fcntl.h>
#include <cerrno>

using namespace libplasticfs;


output_file::~output_file()
{
    flush();
    if (fd >= 0)
    {
	fs->close(fd);
	fd = -1;
    }
    fs = 0;
}


output_file::output_file(plasticfs *filesys, const rcstring &filename) :
    fs(filesys),
    fn(filename),
    fd(-1)
{
    fd = fs->open(fn.c_str(), O_WRONLY | O_CREAT | O_TRUNC, 0666);
}


bool
output_file::overflow(const void *buffer, size_t size)
{
    if (fd < 0)
	return false;
    ssize_t result = fs->write(fd, buffer, size);
    if (result < 0)
	return false;
    if ((size_t)result != size)
    {
	errno = EIO;
	return false;
    }
    return true;
}
