//
//	plasticfs - the plastic file system
//	Copyright (C) 2003 Peter Miller;
//	All rights reserved.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
//
// MANIFEST: interface definition for lib/output/file.cc
//

#ifndef LIB_OUTPUT_FILE_H
#define LIB_OUTPUT_FILE_H

#pragma interface "libplasticfs::output_file"

#include <output.h>
#include <plasticfs.h>

namespace libplasticfs {

/**
  * The output_file class is used to represent
  */
class output_file:
    public output
{
public:
    /**
      * The destructor.
      */
    virtual ~output_file();

    /**
      * The constructor.
      */
    output_file(plasticfs *filesys, const rcstring &filename);

    // See base class for documentation.
    bool overflow(const void *, size_t);

private:
    /**
      * The fs instance variable is used to remember which file system
      * to use to write the file to.
      */
    plasticfs *fs;

    /**
      * The fn instance variable is used to remember which file to write
      * the data to.
      */
    rcstring fn;

    /**
      * The fd instance variable is used to remember the file descriptor
      * when the file is open.  It is -1 when the file is closed.
      */
    int fd;

    /**
      * The default constructor.  Do not use.
      */
    output_file();

    /**
      * The copy constructor.  Do not use.
      */
    output_file(const output_file &);

    /**
      * The assignment operator.  Do not use.
      */
    output_file &operator=(const output_file &);
};

} // namespace libplasticfs

#endif // LIB_OUTPUT_FILE_H
