//
//	plasticfs - the plastic file system
//	Copyright (C) 2002, 2003 Peter Miller;
//	All rights reserved.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
//
// MANIFEST: interface definition for lib/plasticfs/dlsym_rtld_next.cc
//

#ifndef LIB_PLASTICFS_DLSYM_RTLD_NEXT_H
#define LIB_PLASTICFS_DLSYM_RTLD_NEXT_H

#pragma interface "libplasticfs::plasticfs_dlsym_rtld_next"

#include <plasticfs.h>

//
// Define the DLSYM_OPENDIR symbol if you want to pass through to libc's
// opendir family, rather than use plasticfs's implementation.
//
// #define DLSYM_OPENDIR

namespace libplasticfs {

/**
  * The plasticfs_dlsym_rtld_next class is used to represent the file
  * system interface of the next module along the LD_PRELOAD list and
  * the LD_LIBRARY_PATH, usually the C library.
  */
class plasticfs_dlsym_rtld_next:
    public plasticfs
{
public:
    /**
      * The destructor.
      */
    virtual ~plasticfs_dlsym_rtld_next();

    /**
      * The default constructor.
      */
    plasticfs_dlsym_rtld_next();

    // See base class for documentation.
    int open(const char *open, int flags, int mode);

    // See base class for documentation.
    int close(int filedesc);

    // See base class for documentation.
    ssize_t read(int, void *, size_t);

    // See base class for documentation.
    ssize_t write(int, const void *, size_t);

    // See base class for documentation.
    off_t lseek(int, off_t, int);

    // See base class for documentation.
    off64_t lseek64(int, off64_t, int);

    // See base class for documentation.
    int readlink(const char *, char *, size_t);

    // See base class for documentation.
    int xstat(int, const char *, struct stat *);

    // See base class for documentation.
    int xstat64(int, const char *, struct stat64 *);

    // See base class for documentation.
    int lxstat(int, const char *, struct stat *);

    // See base class for documentation.
    int lxstat64(int, const char *, struct stat64 *);

    // See base class for documentation.
    int fxstat(int, int filedesc, struct stat *);

    // See base class for documentation.
    int fxstat64(int, int filedesc, struct stat64 *);

#ifdef DLSYM_OPENDIR
    // See base class for documentation.
    DIR *opendir(const char *);

    // See base class for documentation.
    int closedir(DIR *);

    // See base class for documentation.
    dirent *readdir(DIR *);

    // See base class for documentation.
    dirent64 *readdir64(DIR *);

    // See base class for documentation.
    void rewinddir(DIR *);

    // See base class for documentation.
    void seekdir(DIR *, off_t);

    // See base class for documentation.
    off_t telldir(DIR *);

    // See base class for documentation.
    int dirfd(DIR *);

    // See base class for documentation.
    char *getpwd(char *, long);
#endif // DLSYM_OPENDIR

    // See base class for documentation.
    int getdents(int, dirent *, unsigned);

    // See base class for documentation.
    int getdents64(int, dirent64 *, unsigned);

    // See base class for documentation.
    ssize_t getdirentries(int, char *, size_t, off_t *);

    // See base class for documentation.
    ssize_t getdirentries64(int, char *, size_t, off64_t *);

    // See base class for documentation.
    int chmod(const char *, mode_t);

    // See base class for documentation.
    int fchmod(int, mode_t);

    // See base class for documentation.
    int mkdir(const char *, mode_t);

    // See base class for documentation.
    int mkfifo(const char *, mode_t);

    // See base class for documentation.
    int unlink(const char *);

    // See base class for documentation.
    int rename(const char *, const char *);

    // See base class for documentation.
    int link(const char *, const char *);

    // See base class for documentation.
    long pathconf(const char *, int);

    // See base class for documentation.
    long fpathconf(int, int);

    // See base class for documentation.
    int xmknod(int ver, const char *path, mode_t mode, dev_t dev);

    // See base class for documentation.
    int fcntl(int, int, void *);

    // See base class for documentation.
    int fcntl64(int, int, void *);

    // See base class for documentation.
    int dup(int);

    // See base class for documentation.
    int dup2(int, int);

    // See base class for documentation.
    int symlink(const char *, const char *);

    // See base class for documentation.
    int chdir(const char *);

    // See base class for documentation.
    int fchdir(int);

    // See base class for documentation.
    int rmdir(const char *);

    // See base class for documentation.
    int utime(const char *, utimbuf *);

    // See base class for documentation.
    int lutime(const char *, utimbuf *);

    // See base class for documentation.
    int futime(int, utimbuf *);

    // See base class for documentation.
    int execve(const char *, char *const *, char *const *);

    // See base class for documentation.
    int fexecve(int, char *const *, char *const *);

    // See base class for documentation.
    int access(const char *, int);

    // See base class for documentation.
    int statfs(const char *, struct statfs *);

    // See base class for documentation.
    int fstatfs(int, struct statfs *);

    // See base class for documentation.
    int statfs64(const char *, struct statfs64 *);

    // See base class for documentation.
    int fstatfs64(int, struct statfs64 *);

    // See base class for documentation.
    int truncate(const char *, off_t);

    // See base class for documentation.
    int truncate64(const char *, off64_t);

    // See base class for documentation.
    int ftruncate(int, off_t);

    // See base class for documentation.
    int ftruncate64(int, off64_t);

private:
    /**
      * The find_symbol method is used to find the named symbol along
      * the LD_LIBRARY_PATH.  Returns NULL if not found.
      */
    void *find_symbol(const char *name);

    /**
      * The report_missing_symbol method is used to issue an error
      * message to complain about a missing symbol.
      */
    void report_missing_symbol(const char *);

    /**
      * The open_t type is used to declare pointers to open functions.
      */
    typedef int (*open_t)(const char *path, int flags, ...);

    /**
      * The open_intercepted instance variable is used to remember the
      * value of the open symbol before we intercepted it.
      */
    open_t open_intercepted;

    /**
      * The close_t type is used to declare pointers to close functions.
      */
    typedef int (*close_t)(int filedesc);

    /**
      * The close_intercepted instance variable is used to remember the
      * value of the close symbol before we intercepted it.
      */
    close_t close_intercepted;

    /**
      * The read_t type is used to declare pointers to read functions.
      */
    typedef ssize_t (*read_t)(int filedesc, void *data, size_t nbytes);

    /**
      * The read_intercepted instance variable is used to remember the
      * value of the read symbol before we intercepted it.
      */
    read_t read_intercepted;

    /**
      * The write_t type is used to declare pointers to write functions.
      */
    typedef ssize_t (*write_t)(int filedesc, const void *data, size_t nbytes);

    /**
      * The write_intercepted instance variable is used to remember the
      * value of the write symbol before we intercepted it.
      */
    write_t write_intercepted;

    /**
      * The lseek_t type is used to declare pointers to lseek functions.
      */
    typedef off_t (*lseek_t)(int filedesc, off_t offset, int whence);

    /**
      * The lseek_intercepted instance variable is used to remember the
      * value of the lseek symbol before we intercepted it.
      */
    lseek_t lseek_intercepted;

    /**
      * The lseek64_t type is used to declare pointers to lseek functions.
      */
    typedef off64_t (*lseek64_t)(int filedesc, off64_t offset, int whence);

    /**
      * The lseek64_intercepted instance variable is used to remember the
      * value of the lseek64 symbol before we intercepted it.
      */
    lseek64_t lseek64_intercepted;

    /**
      * The readlink_t type is used to declare pointers to readlink functions.
      */
    typedef int (*readlink_t)(const char *path, char *buffer, size_t nbytes);

    /**
      * The readlink_intercepted instance variable is used to remember the
      * value of the readlink symbol before we intercepted it.
      */
    readlink_t readlink_intercepted;

    /**
      * The xstat_t type is used to declare pointers to __xstat functions.
      */
    typedef int (*xstat_t)(int, const char *, struct stat *);

    /**
      * The xstat_intercepted instance variable is used to remember the
      * value of the __xstat symbol before we intercepted it.
      */
    xstat_t xstat_intercepted;

    /**
      * The xstat64_t type is used to declare pointers to __xstat64 functions.
      */
    typedef int (*xstat64_t)(int, const char *, struct stat64 *);

    /**
      * The xstat64_intercepted instance variable is used to remember the
      * value of the __xstat64 symbol before we intercepted it.
      */
    xstat64_t xstat64_intercepted;

    /**
      * The lxstat_intercepted instance variable is used to remember the
      * value of the __lxstat symbol before we intercepted it.
      */
    xstat_t lxstat_intercepted;

    /**
      * The lxstat64_intercepted instance variable is used to remember the
      * value of the __lxstat64 symbol before we intercepted it.
      */
    xstat64_t lxstat64_intercepted;

    /**
      * The fxstat_t type is used to declare pointers to __fxstat functions.
      */
    typedef int (*fxstat_t)(int, int, struct stat *);

    /**
      * The fxstat_intercepted instance variable is used to remember the
      * value of the __fxstat symbol before we intercepted it.
      */
    fxstat_t fxstat_intercepted;

    /**
      * The fxstat64_t type is used to declare pointers to __fxstat64 functions.
      */
    typedef int (*fxstat64_t)(int, int, struct stat64 *);

    /**
      * The fxstat64_intercepted instance variable is used to remember the
      * value of the __fxstat64 symbol before we intercepted it.
      */
    fxstat64_t fxstat64_intercepted;

#ifdef DLSYM_OPENDIR
    /**
      * The opendir_t type is used to declare pointers to opendir functions.
      */
    typedef DIR *(*opendir_t)(const char *);

    /**
      * The opendir_intercepted instance variable is used to remember the
      * value of the opendir symbol before we intercepted it.
      */
    opendir_t opendir_intercepted;

    /**
      * The closedir_t type is used to declare pointers to closedir functions.
      */
    typedef int (*closedir_t)(DIR *);

    /**
      * The closedir_intercepted instance variable is used to remember the
      * value of the closedir symbol before we intercepted it.
      */
    closedir_t closedir_intercepted;

    /**
      * The readdir_t type is used to declare pointers to readdir functions.
      */
    typedef dirent *(*readdir_t)(DIR *);

    /**
      * The readdir_intercepted instance variable is used to remember the
      * value of the readdir symbol before we intercepted it.
      */
    readdir_t readdir_intercepted;

    /**
      * The readdir64_t type is used to declare pointers to readdir64
      * functions.
      */
    typedef dirent64 *(*readdir64_t)(DIR *);

    /**
      * The readdir64_intercepted instance variable is used to remember
      * the value of the readdir64 symbol before we intercepted it.
      */
    readdir64_t readdir64_intercepted;

    /**
      * The rewinddir_t type is used to declare pointers to rewinddir functions.
      */
    typedef void (*rewinddir_t)(DIR *);

    /**
      * The rewinddir_intercepted instance variable is used to remember the
      * value of the rewinddir symbol before we intercepted it.
      */
    rewinddir_t rewinddir_intercepted;

    /**
      * The seekdir_t type is used to declare pointers to seekdir functions.
      */
    typedef void (*seekdir_t)(DIR *, off_t);

    /**
      * The seekdir_intercepted instance variable is used to remember the
      * value of the seekdir symbol before we intercepted it.
      */
    seekdir_t seekdir_intercepted;

    /**
      * The telldir_t type is used to declare pointers to telldir functions.
      */
    typedef off_t (*telldir_t)(DIR *);

    /**
      * The telldir_intercepted instance variable is used to remember the
      * value of the telldir symbol before we intercepted it.
      */
    telldir_t telldir_intercepted;

    /**
      * The dirfd_t type is used to declare pointers to dirfd functions.
      */
    typedef int (*dirfd_t)(DIR *);

    /**
      * The dirfd_intercepted instance variable is used to remember the
      * value of the dirfd symbol before we intercepted it.
      */
    dirfd_t dirfd_intercepted;

    /**
      * The getcwd_t type is used to declare pointers to getcwd functions.
      */
    typedef char *(*getcwd_t)(char *, size_t);

    /**
      * The getcwd_intercepted instance variable is used to remember the
      * value of the getcwd symbol before we intercepted it.
      */
    getcwd_t getcwd_intercepted;
#endif // DLSYM_OPENDIR

    /**
      * The getdents_t type is used to declare pointers to
      * getdents functions.
      */
    typedef int (*getdents_t)(int, dirent *, unsigned);

    /**
      * The getdents_intercepted instance variable is used to
      * remember the value of the getdents symbol before we
      * intercepted it.
      */
    getdents_t getdents_intercepted;

    /**
      * The getdents64_t type is used to declare pointers to
      * getdents64 functions.
      */
    typedef int (*getdents64_t)(int, dirent64 *, unsigned);

    /**
      * The getdents64_intercepted instance variable is used to
      * remember the value of the getdents64 symbol before we
      * intercepted it.
      */
    getdents64_t getdents64_intercepted;

    /**
      * The getdirentries_t type is used to declare pointers to
      * getdirentries functions.
      */
    typedef ssize_t (*getdirentries_t)(int, char *, size_t, off_t *);

    /**
      * The getdirentries_intercepted instance variable is used to
      * remember the value of the getdirentries symbol before we
      * intercepted it.
      */
    getdirentries_t getdirentries_intercepted;

    /**
      * The getdirentries64_t type is used to declare pointers to
      * getdirentries64 functions.
      */
    typedef ssize_t (*getdirentries64_t)(int, char *, size_t, off64_t *);

    /**
      * The getdirentries64_intercepted instance variable is used to
      * remember the value of the getdirentries64 symbol before we
      * intercepted it.
      */
    getdirentries64_t getdirentries64_intercepted;

    /**
      * The chmod_t type is used to declare pointers to
      * chmod functions.
      */
    typedef int (*chmod_t)(const char *, mode_t);

    /**
      * The chmod_intercepted instance variable is used to
      * remember the value of the chmod symbol before we
      * intercepted it.
      */
    chmod_t chmod_intercepted;

    /**
      * The fchmod_t type is used to declare pointers to
      * fchmod functions.
      */
    typedef int (*fchmod_t)(int, mode_t);

    /**
      * The fchmod_intercepted instance variable is used to
      * remember the value of the fchmod symbol before we
      * intercepted it.
      */
    fchmod_t fchmod_intercepted;

    /**
      * The mkdir_intercepted instance variable is used to
      * remember the value of the mkdir symbol before we
      * intercepted it.
      */
    chmod_t mkdir_intercepted;

    /**
      * The mkfifo_intercepted instance variable is used to
      * remember the value of the mkfifo symbol before we
      * intercepted it.
      */
    chmod_t mkfifo_intercepted;

    /**
      * The unlink_t type is used to declare pointers to
      * unlink functions.
      */
    typedef int (*unlink_t)(const char *);

    /**
      * The unlink_intercepted instance variable is used to
      * remember the value of the unlink symbol before we
      * intercepted it.
      */
    unlink_t unlink_intercepted;

    /**
      * The rename_t type is used to declare pointers to
      * rename functions.
      */
    typedef int (*rename_t)(const char *, const char *);

    /**
      * The rename_intercepted instance variable is used to
      * remember the value of the rename symbol before we
      * intercepted it.
      */
    rename_t rename_intercepted;

    /**
      * The link_intercepted instance variable is used to
      * remember the value of the link symbol before we
      * intercepted it.
      */
    rename_t link_intercepted;

    /**
      * The symlink_intercepted instance variable is used to remember
      * the value of the symlink symbol before we intercepted it.
      */
    rename_t symlink_intercepted;

    /**
      * The pathconf_t type is used to declare pointers to
      * pathconf functions.
      */
    typedef long (*pathconf_t)(const char *, int);

    /**
      * The pathconf_intercepted instance variable is used to
      * remember the value of the pathconf symbol before we
      * intercepted it.
      */
    pathconf_t pathconf_intercepted;

    /**
      * The fpathconf_t type is used to declare pointers to
      * fpathconf functions.
      */
    typedef long (*fpathconf_t)(int, int);

    /**
      * The fpathconf_intercepted instance variable is used to
      * remember the value of the fpathconf symbol before we
      * intercepted it.
      */
    fpathconf_t fpathconf_intercepted;

    /**
      * The xmknod_t type is used to declare pointers to xmknod functions.
      */
    typedef int (*xmknod_t)(int, const char *, mode_t, dev_t);

    /**
      * The xmknod_intercepted instance variable is used to remember
      * the value of the xmknod symbol before we intercepted it.
      */
    xmknod_t xmknod_intercepted;

    /**
      * The fcntl_t type is used to declare pointers to fcntl functions.
      */
    typedef int (*fcntl_t)(int, int, ...);

    /**
      * The fcntl_intercepted instance variable is used to remember
      * the value of the fcntl symbol before we intercepted it.
      */
    fcntl_t fcntl_intercepted;

    /**
      * The fcntl64_intercepted instance variable is used to remember
      * the value of the fcntl64 symbol before we intercepted it.
      */
    fcntl_t fcntl64_intercepted;

    /**
      * The dup_t type is used to declare pointers to dup functions.
      */
    typedef int (*dup_t)(int);

    /**
      * The dup_intercepted instance variable is used to remember
      * the value of the dup symbol before we intercepted it.
      */
    dup_t dup_intercepted;

    /**
      * The dup2_t type is used to declare pointers to dup2 functions.
      */
    typedef int (*dup2_t)(int, int);

    /**
      * The dup2_intercepted instance variable is used to remember
      * the value of the dup2 symbol before we intercepted it.
      */
    dup2_t dup2_intercepted;

    /**
      * The chdir_t type is used to declare pointers to chdir functions.
      */
    typedef int (*chdir_t)(const char *);

    /**
      * The chdir_intercepted instance variable is used to remember
      * the value of the chdir symbol before we intercepted it.
      */
    chdir_t chdir_intercepted;

    /**
      * The fchdir_t type is used to declare pointers to fchdir functions.
      */
    typedef int (*fchdir_t)(int);

    /**
      * The fchdir_intercepted instance variable is used to remember
      * the value of the fchdir symbol before we intercepted it.
      */
    fchdir_t fchdir_intercepted;

    /**
      * The rmdir_t type is used to declare pointers to rmdir functions.
      */
    typedef int (*rmdir_t)(const char *);

    /**
      * The rmdir_intercepted instance variable is used to remember
      * the value of the rmdir symbol before we intercepted it.
      */
    rmdir_t rmdir_intercepted;

    /**
      * The utime_t type is used to declare pointers to utime functions.
      */
    typedef int (*utime_t)(const char *, utimbuf *);

    /**
      * The utime_intercepted instance variable is used to remember
      * the value of the utime symbol before we intercepted it.
      */
    utime_t utime_intercepted;

    /**
      * The lutime_intercepted instance variable is used to remember
      * the value of the lutime symbol before we intercepted it.
      */
    utime_t lutime_intercepted;

    /**
      * The futime_t type is used to declare pointers to futime functions.
      */
    typedef int (*futime_t)(int, utimbuf *);

    /**
      * The futime_intercepted instance variable is used to remember
      * the value of the futime symbol before we intercepted it.
      */
    futime_t futime_intercepted;

    /**
      * The execve_t type is used to declare pointers to execve functions.
      */
    typedef int (*execve_t)(const char *, char *const *, char *const *);

    /**
      * The execve_intercepted instance variable is used to remember
      * the value of the execve symbol before we intercepted it.
      */
    execve_t execve_intercepted;

    /**
      * The fexecve_t type is used to declare pointers to fexecve functions.
      */
    typedef int (*fexecve_t)(int, char *const *, char *const *);

    /**
      * The fexecve_intercepted instance variable is used to remember
      * the value of the fexecve symbol before we intercepted it.
      */
    fexecve_t fexecve_intercepted;

    /**
      * The access_t type is used to declare pointers to access functions.
      */
    typedef int (*access_t)(const char *, int);

    /**
      * The access_intercepted instance variable is used to remember
      * the value of the access symbol before we intercepted it.
      */
    access_t access_intercepted;

    /**
      * The statfs_t type is used to declare pointers to statfs functions.
      */
    typedef int (*statfs_t)(const char *, struct statfs *);

    /**
      * The statfs_intercepted instance variable is used to remember
      * the value of the statfs symbol before we intercepted it.
      */
    statfs_t statfs_intercepted;

    /**
      * The fstatfs_t type is used to declare pointers to fstatfs functions.
      */
    typedef int (*fstatfs_t)(int, struct statfs *);

    /**
      * The fstatfs_intercepted instance variable is used to remember
      * the value of the fstatfs symbol before we intercepted it.
      */
    fstatfs_t fstatfs_intercepted;

    /**
      * The statfs64_t type is used to declare pointers to statfs64 functions.
      */
    typedef int (*statfs64_t)(const char *, struct statfs64 *);

    /**
      * The statfs64_intercepted instance variable is used to remember
      * the value of the statfs64 symbol before we intercepted it.
      */
    statfs64_t statfs64_intercepted;

    /**
      * The fstatfs64_t type is used to declare pointers to fstatfs64 functions.
      */
    typedef int (*fstatfs64_t)(int, struct statfs64 *);

    /**
      * The fstatfs64_intercepted instance variable is used to remember
      * the value of the fstatfs64 symbol before we intercepted it.
      */
    fstatfs64_t fstatfs64_intercepted;

    /**
      * The truncate_t type is used to declare pointers to truncate functions.
      */
    typedef int (*truncate_t)(const char *, off_t);

    /**
      * The truncate_intercepted instance variable is used to remember
      * the value of the truncate symbol before we intercepted it.
      */
    truncate_t truncate_intercepted;

    /**
      * The ftruncate_t type is used to declare pointers to ftruncate functions.
      */
    typedef int (*ftruncate_t)(int, off_t);

    /**
      * The ftruncate_intercepted instance variable is used to remember
      * the value of the ftruncate symbol before we intercepted it.
      */
    ftruncate_t ftruncate_intercepted;

    /**
      * The truncate64_t type is used to declare pointers to truncate64
      * functions.
      */
    typedef int (*truncate64_t)(const char *, off64_t);

    /**
      * The truncate64_intercepted instance variable is used to remember
      * the value of the truncate64 symbol before we intercepted it.
      */
    truncate64_t truncate64_intercepted;

    /**
      * The ftruncate64_t type is used to declare pointers to ftruncate64
      * functions.
      */
    typedef int (*ftruncate64_t)(int, off64_t);

    /**
      * The ftruncate64_intercepted instance variable is used to remember
      * the value of the ftruncate64 symbol before we intercepted it.
      */
    ftruncate64_t ftruncate64_intercepted;

    /**
      * The copy constructor.  Do not use.
      */
    plasticfs_dlsym_rtld_next(const plasticfs_dlsym_rtld_next &);

    /**
      * The assignment operator.  Do not use.
      */
    plasticfs_dlsym_rtld_next &operator=(const plasticfs_dlsym_rtld_next &);
};

} // namespace libplasticfs

#endif // LIB_PLASTICFS_DLSYM_RTLD_NEXT_H
