//
//	plasticfs - the plastic file system
//	Copyright (C) 2002, 2003 Peter Miller;
//	All rights reserved.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
//
// MANIFEST: functions to impliment the plasticfs_error_message class
//

#include <plasticfs.h>
#include <errno_insulator.h>
#include <cerrno>
#include <cstdarg>
#include <cstdio>
#include <cstring>

using namespace libplasticfs;

//
// We have to skip all of the libc interfacing and go direct to the kernel
// if we want any hope of guaranteeing to get the message to the user.
// (Otherwise we risk recursing back into plasticfs.)
//
// The only problem is, the asm code that gets inlined can't cope
// with -fPIC.  I have no idea how glibc manages this trick.
//
#if 0
#include <sys/types.h>
#include <sys/syscall.h>
static _syscall3(int, write, int, fd, const char *, buf, off_t, count)
#else
#include <unistd.h>
#endif


void
plasticfs::error_message(const char *fmt, ...)
{
    errno_insulator guard;
    static int double_jeopardy;

    va_list ap;
    va_start(ap, fmt);
    char buffer[1024];
    vsnprintf(buffer, sizeof(buffer), fmt, ap);
    va_end(ap);

    if (!double_jeopardy++)
	::write(2, buffer, strlen(buffer));
    --double_jeopardy;
}
