//
//	plasticfs - the plastic file system
//	Copyright (C) 2003 Peter Miller;
//	All rights reserved.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
//
// MANIFEST: interface definition for lib/plasticfs/filter/module.cc
//

#ifndef LIB_PLASTICFS_FILTER_MODULE_H
#define LIB_PLASTICFS_FILTER_MODULE_H

#pragma interface "libplasticfs::plasticfs_filter_module"

#include <plasticfs/filter.h>
#include <rcstring_list.h>

namespace libplasticfs {

/**
  * The plasticfs_filter_module class is used to represent a loadable
  * module being used as a file system filter.
  */
class plasticfs_filter_module:
    public plasticfs_filter
{
public:
    /**
      * The destructor.
      */
    virtual ~plasticfs_filter_module();

    /**
      * The factory method is used to craete a new instance of the
      * module filter.  Note that it will return NULL if the module
      * can't be found, or has some other error.
      */
    static plasticfs *factory(plasticfs *deeper, const rcstring_list &args);

    // See base class for documentation.
    bool is_below_mount_point(const char *, size_t &);

    // See base class for documentation.
    int relative_readlink(const char *, char *, size_t);

    // See base class for documentation.
    int relative_lstat(const char *, struct stat *);

protected:
    /**
      * The constructor.
      */
    plasticfs_filter_module(plasticfs *deeper, void *handle);

private:
    /**
      * The handle instance variable is used to remember the opaque
      * handle to the loadable module being used.
      */
    void *handle;

    /**
      * The default constructor.  Do not use.
      */
    plasticfs_filter_module();

    /**
      * The copy constructor.  Do not use.
      */
    plasticfs_filter_module(const plasticfs_filter_module &);

    /**
      * The assignment operator.  Do not use.
      */
    plasticfs_filter_module &operator=(const plasticfs_filter_module &);
};

} // namespace libplasticfs

#endif // LIB_PLASTICFS_FILTER_MODULE_H
