//
//	plasticfs - the plastic file system
//	Copyright (C) 2003 Peter Miller;
//	All rights reserved.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
//
// MANIFEST: interface definition for lib/plasticfs/filter/nocase/dos.cc
//

#ifndef LIB_PLASTICFS_FILTER_NOCASE_DOS_H
#define LIB_PLASTICFS_FILTER_NOCASE_DOS_H

#pragma interface "libplasticfs::plasticfs_filter_nocase_dos"

#include <plasticfs/filter/nocase.h>

namespace libplasticfs {

/**
  * The plasticfs_filter_nocase_dos class is used to represent a DOS
  * 8.3 file system, where directory reads are mapped to upper case
  * (and worse).
  */
class plasticfs_filter_nocase_dos:
    public plasticfs_filter_nocase
{
public:
    /**
      * The destructor.
      */
    virtual ~plasticfs_filter_nocase_dos();

    /**
      * The constructor.
      */
    plasticfs_filter_nocase_dos(plasticfs *, const rcstring_list &);

    // See base class for documentation.
    bool relative_name_mapping(const char *, rcstring &);

    // See base class for documentation.
    void relative_name_unmapping(char *, size_t);

    // See base class for documentation.
    int pc_name_max(int);

    // See base class for documentation.
    int pc_no_trunc(int);

    /**
      * The factory method is used to create new instances of this class.
      */
    static plasticfs *factory(plasticfs *, const rcstring_list &);

private:
    /**
      * The default constructor.  Do not use.
      */
    plasticfs_filter_nocase_dos();

    /**
      * The copy constructor.  Do not use.
      */
    plasticfs_filter_nocase_dos(const plasticfs_filter_nocase_dos &);

    /**
      * The assignment operator.  Do not use.
      */
    plasticfs_filter_nocase_dos &operator=(const plasticfs_filter_nocase_dos &);
};

} // namespace libplasticfs

#endif // LIB_PLASTICFS_FILTER_NOCASE_DOS_H
