//
//	plasticfs - the plastic file system
//	Copyright (C) 2003 Peter Miller;
//	All rights reserved.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
//
// MANIFEST: interface definition for lib/plasticfs/filter/smartlink.cc
//

#ifndef LIB_PLASTICFS_FILTER_SMARTLINK_H
#define LIB_PLASTICFS_FILTER_SMARTLINK_H

#pragma interface "libplasticfs::plasticfs_filter_smartlink"

#include <plasticfs/filter.h>
#include <rcstring_list.h>

namespace libplasticfs {

/**
  * The plasticfs_filter_smartlink class is used to represent a file
  * system filter which lets you embed environment variables into
  * symbolic links.
  */
class plasticfs_filter_smartlink:
    public plasticfs_filter
{
public:
    /**
      * The destructor.
      */
    virtual ~plasticfs_filter_smartlink();

    /**
      * The constructor.
      */
    plasticfs_filter_smartlink(plasticfs *deeper, const rcstring_list &args);

    // See base class for documentation.
    int open(const char *path, int flags, int mode);

    // See base class for documentation.
    int xstat(int, const char *, struct stat *);

    // See base class for documentation.
    int xstat64(int, const char *, struct stat64 *);

    // See base class for documentation.
    int lxstat(int, const char *, struct stat *);

    // See base class for documentation.
    int lxstat64(int, const char *, struct stat64 *);

    // See base class for documentation.
    int readlink(const char *, char *, size_t);

    // See base class for documentation.
    int chmod(const char *, mode_t);

    // See base class for documentation.
    int mkdir(const char *, mode_t);

    // See base class for documentation.
    int mkfifo(const char *, mode_t);

    // See base class for documentation.
    int unlink(const char *);

    // See base class for documentation.
    int rename(const char *, const char *);

    // See base class for documentation.
    int link(const char *, const char *);

    // See base class for documentation.
    long pathconf(const char *, int);

    // See base class for documentation.
    int xmknod(int ver, const char *path, mode_t mode, dev_t dev);

    // See base class for documentation.
    int symlink(const char *, const char *);

    // See base class for documentation.
    int chdir(const char *);

    // See base class for documentation.
    int rmdir(const char *);

    // See base class for documentation.
    int utime(const char *, utimbuf *);

    // See base class for documentation.
    int lutime(const char *, utimbuf *);

    // See base class for documentation.
    int execve(const char *, char *const *, char *const *);

    // See base class for documentation.
    int access(const char *, int);

    // See base class for documentation.
    int statfs(const char *, struct statfs *);

    // See base class for documentation.
    int statfs64(const char *, struct statfs64 *);

    // See base class for documentation.
    int truncate(const char *, off_t);

    // See base class for documentation.
    int truncate64(const char *, off64_t);

    /**
      * The factory method is used to create anew instance of this class.
      */
    static plasticfs *factory(plasticfs *, const rcstring_list &);

private:
    typedef plasticfs_filter inherited;

    /**
      * The mount_point instance variable is used to remember the point
      * below which symbolic links are to be expanded.
      */
    rcstring mount_point;

    // See base class for documentation.
    bool is_below_mount_point(const char *, size_t &);

    // See base class for documentation.
    int relative_readlink(const char *, char *, size_t);

    // See base class for documentation.
    int relative_lstat(const char *, struct stat *);

    /**
      * The default constructor.  Do not use.
      */
    plasticfs_filter_smartlink();

    /**
      * The copy constructor.  Do not use.
      */
    plasticfs_filter_smartlink(const plasticfs_filter_smartlink &);

    /**
      * The assignment operator.  Do not use.
      */
    plasticfs_filter_smartlink &operator=(const plasticfs_filter_smartlink &);
};

} // namespace libplasticfs

#endif // LIB_PLASTICFS_FILTER_SMARTLINK_H
