//
//	plasticfs - the plastic file system
//	Copyright (C) 2002, 2003 Peter Miller;
//	All rights reserved.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
//
// MANIFEST: functions to impliment the plasticfs_libc class
//

#pragma implementation "libplasticfs::plasticfs_libc"

#include <plasticfs/libc.h>

using namespace libplasticfs;


plasticfs_libc::~plasticfs_libc()
{
}


plasticfs_libc::plasticfs_libc() :
    plasticfs()
{
}


extern "C" {
int __libc_open(const char *, int, int);
int __libc_open64(const char *, int, int);
int __libc_close(int);
ssize_t __libc_read(int, void *, size_t);
ssize_t __libc_write(int, const void *, size_t);
int __libc_readlink(const char *, char *, size_t);
int __libc_xstat(int, const char *, struct stat *);
int __libc_xstat64(int, const char *, struct stat64 *);
int __libc_lxstat(int, const char *, struct stat *);
int __libc_lxstat64(int, const char *, struct stat64 *);
int __libc_fxstat(int, int filedesc, struct stat *);
int __libc_fxstat64(int, int filedesc, struct stat64 *);
off_t __libc_lseek(int, off_t, int);
off64_t __libc_lseek64(int, off64_t, int);
int __libc_getdents(int, dirent *, unsigned);
int __libc_getdents64(int, dirent64 *, unsigned);
ssize_t __libc_getdirentries(int, char *, size_t, off_t *);
ssize_t __libc_getdirentries64(int, char *, size_t, off64_t *);
int __libc_chmod(const char *, mode_t);
int __libc_fchmod(int, mode_t);
int __libc_mkdir(const char *, mode_t);
int __libc_mkfifo(const char *, mode_t);
int __libc_unlink(const char *);
int __libc_rename(const char *, const char *);
int __libc_link(const char *, const char *);
long __libc_pathconf(const char *, int);
long __libc_fpathconf(int, int);
}


int
plasticfs_libc::open(const char *path, int flags, int mode)
{
    return __libc_open(path, flags, mode);
}


int
plasticfs_libc::close(int filedesc)
{
    return __libc_close(filedesc);
}


ssize_t
plasticfs_libc::read(int filedesc, void *data, size_t nbytes)
{
    return __libc_read(filedesc, data, nbytes);
}


ssize_t
plasticfs_libc::write(int filedesc, const void *data, size_t nbytes)
{
    return __libc_write(filedesc, data, nbytes);
}


int
plasticfs_libc::readlink(const char *path, char *data, size_t nbytes)
{
    return __libc_readlink(path, data, nbytes);
}


int
plasticfs_libc::xstat(int version, const char *path, struct stat *buf)
{
    return __libc_xstat(version, path, buf);
}


int
plasticfs_libc::xstat64(int version, const char *path, struct stat64 *buf)
{
    return __libc_xstat64(version, path, buf);
}


int
plasticfs_libc::lxstat(int version, const char *path, struct stat *buf)
{
    return __libc_lxstat(version, path, buf);
}


int
plasticfs_libc::lxstat64(int version, const char *path, struct stat64 *buf)
{
    return __libc_lxstat64(version, path, buf);
}


int
plasticfs_libc::fxstat(int version, int filedesc, struct stat *buf)
{
    return __libc_fxstat(version, filedesc, buf);
}


int
plasticfs_libc::fxstat64(int version, int filedesc, struct stat64 *buf)
{
    return __libc_fxstat64(version, filedesc, buf);
}


off_t
plasticfs_libc::lseek(int fd, off_t offset, int whence)
{
    return __libc_lseek(fd, offset, whence);
}


off64_t
plasticfs_libc::lseek64(int fd, off64_t offset, int whence)
{
    return __libc_lseek64(fd, offset, whence);
}


int 
plasticfs_libc::getdents(int fd, dirent *buf, unsigned nbytes)
{
    return __libc_getdents(fd, buf, nbytes);
}


int 
plasticfs_libc::getdents64(int fd, dirent64 *buf, unsigned nbytes)
{
    return __libc_getdents64(fd, buf, nbytes);
}


ssize_t 
plasticfs_libc::getdirentries(int fd, char *buf, size_t nbytes, off_t *base)
{
    return __libc_getdirentries(fd, buf, nbytes, base);
}


ssize_t 
plasticfs_libc::getdirentries64(int fd, char *buf, size_t nbytes, off64_t *base)
{
    return __libc_getdirentries64(fd, buf, nbytes, base);
}


int
plasticfs_libc::chmod(const char *path, mode_t mode)
{
    return __libc_chmod(path, mode);
}


int
plasticfs_libc::fchmod(int fd, mode_t mode)
{
    return __libc_fchmod(fd, mode);
}


int
plasticfs_libc::mkdir(const char *path, mode_t mode)
{
    return __libc_mkdir(path, mode);
}


int
plasticfs_libc::mkfifo(const char *path, mode_t mode)
{
    return __libc_mkfifo(path, mode);
}


int
plasticfs_libc::unlink(const char *path)
{
    return __libc_unlink(path);
}


int
plasticfs_libc::rename(const char *oldpath, const char *newpath)
{
    return __libc_rename(oldpath, newpath);
}


int
plasticfs_libc::link(const char *oldpath, const char *newpath)
{
    return __libc_link(oldpath, newpath);
}


long
plasticfs_libc::pathconf(const char *path, int name)
{
    return __libc_pathconf(path, name);
}


long
plasticfs_libc::fpathconf(int fd, int name)
{
    return __libc_fpathconf(fd, name);
}
