//
//	plasticfs - the plastic file system
//	Copyright (C) 2002, 2003 Peter Miller;
//	All rights reserved.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
//
// MANIFEST: interface definition for lib/plasticfs/libc.cc
//

#ifndef LIB_PLASTICFS_LIBC_H
#define LIB_PLASTICFS_LIBC_H

#pragma interface "libplasticfs::plasticfs_libc"

#include <plasticfs.h>

namespace libplasticfs {

/**
  * The plasticfs_libc class is used to represent a direct connection
  * to the __libc_* family of functions.  This is only used while we
  * are boot stapping.
  */
class plasticfs_libc:
    public plasticfs
{
public:
    /**
      * The destructor.
      */
    virtual ~plasticfs_libc();

    /**
      * The default constructor.
      */
    plasticfs_libc();

    // See base class for documentation.
    int open(const char *, int, int);

    // See base class for documentation.
    int close(int);

    // See base class for documentation.
    ssize_t read(int, void *, size_t);

    // See base class for documentation.
    ssize_t write(int, const void *, size_t);

    // See base class for documentation.
    int readlink(const char *, char *, size_t);

    // See base class for documentation.
    off_t lseek(int, off_t, int);

    // See base class for documentation.
    off64_t lseek64(int, off64_t, int);

    // See base class for documentation.
    int xstat(int, const char *, struct stat *);

    // See base class for documentation.
    int xstat64(int, const char *, struct stat64 *);

    // See base class for documentation.
    int lxstat(int, const char *, struct stat *);

    // See base class for documentation.
    int lxstat64(int, const char *, struct stat64 *);

    // See base class for documentation.
    int fxstat(int, int filedesc, struct stat *);

    // See base class for documentation.
    int fxstat64(int, int filedesc, struct stat64 *);

    // See base class for documentation.
    int getdents(int, dirent *, unsigned);

    // See base class for documentation.
    int getdents64(int, dirent64 *, unsigned);

    // See base class for documentation.
    ssize_t getdirentries(int, char *, size_t, off_t *);

    // See base class for documentation.
    ssize_t getdirentries64(int, char *, size_t, off64_t *);

    // See base class for documentation.
    int chmod(const char *, mode_t);

    // See base class for documentation.
    int fchmod(int, mode_t);

    // See base class for documentation.
    int mkdir(const char *, mode_t);

    // See base class for documentation.
    int mkfifo(const char *, mode_t);

    // See base class for documentation.
    int unlink(const char *);

    // See base class for documentation.
    int rename(const char *, const char *);

    // See base class for documentation.
    int link(const char *, const char *);

    // See base class for documentation.
    long pathconf(const char *, int);

    // See base class for documentation.
    long fpathconf(int, int);

private:
    /**
      * The copy constructor.  Do not use.
      */
    plasticfs_libc(const plasticfs_libc &);

    /**
      * The assignment operator.  Do not use.
      */
    plasticfs_libc &operator=(const plasticfs_libc &);
};

} // namespace libplasticfs

#endif // LIB_PLASTICFS_LIBC_H
