//
//	plasticfs - the plastic file system
//	Copyright (C) 2002 Peter Miller;
//	All rights reserved.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
//
// MANIFEST: functions to impliment the rcstring_list class
//

#pragma implementation "libplasticfs::rcstring_list"

#include <rcstring_list.h>

using namespace libplasticfs;


rcstring_list::~rcstring_list()
{
    if (list)
	delete [] list;
    list_size = 0;
    list_size_max = 0;
}


rcstring_list::rcstring_list()
{
    list = 0;
    list_size = 0;
    list_size_max = 0;
}


rcstring_list::rcstring_list(const rcstring_list &arg)
{
    list = 0;
    list_size = 0;
    list_size_max = 0;

    while (list_size_max < arg.list_size)
	list_size_max = list_size_max * 2 + 8;
    list = new rcstring[list_size_max];
    for (list_size = 0; list_size < arg.list_size; ++list_size)
	list[list_size] = arg.list[list_size];
}


rcstring_list &
rcstring_list::operator=(const rcstring_list &arg)
{
    if (this != &arg)
    {
	size_t new_list_size_max = list_size_max;
	while (new_list_size_max < arg.list_size)
	    new_list_size_max = new_list_size_max * 2 + 8;
	if (new_list_size_max > list_size_max)
	{
	    delete [] list;
	    list_size_max = new_list_size_max;
	}
	else
	{
	    while (list_size > arg.list_size)
		list[--list_size] = rcstring();
	}
	for (list_size = 0; list_size < arg.list_size; ++list_size)
	    list[list_size] = arg.list[list_size];
    }
    return *this;
}


rcstring
rcstring_list::operator[](size_t n)
    const
{
    if (n >= list_size)
	return rcstring("");
    return list[n];
}


void
rcstring_list::push_back(const rcstring &s)
{
    if (list_size >= list_size_max)
    {
	size_t new_list_size_max = list_size_max * 2 + 8;
	rcstring *new_list = new rcstring[new_list_size_max];
	for (size_t j = 0; j < list_size; ++j)
	    new_list[j] = list[j];
	delete [] list;
	list = new_list;
	list_size_max = new_list_size_max;
    }
    list[list_size++] = s;
}
