//
//	plasticfs - the plastic file system
//	Copyright (C) 2002 Peter Miller;
//	All rights reserved.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
//
// MANIFEST: interface definition for lib/rcstring_list.cc
//

#ifndef LIB_RCSTRING_LIST_H
#define LIB_RCSTRING_LIST_H

#pragma interface "libplasticfs::rcstring_list"

#include <rcstring.h>

namespace libplasticfs {

/**
  * The rcstring_list class is used to represent a list of rcstring
  * values.
  */
class rcstring_list
{
public:
    /**
      * The destructor.
      * Do not derive from this class.
      */
    ~rcstring_list();

    /**
      * The default constructor.
      */
    rcstring_list();

    /**
      * The copy constructor.
      */
    rcstring_list(const rcstring_list &);

    /**
      * The assignment operator.
      */
    rcstring_list &operator=(const rcstring_list &);

    /**
      * The size method is used to determine the number of elements in
      * the list.
      */
    size_t size() const { return list_size; }

    /**
      * The operator[] method is used to extract a specific element
      * of the list.  Note that this list cannot be assigned to this way.
      */
    rcstring operator[](size_t) const;

    /**
      * The push_back method is used to append an element to the back
      * of the list.
      */
    void push_back(const rcstring &);

private:
    rcstring *list;
    size_t list_size;
    size_t list_size_max;
};

} // namespace libplasticfs

#endif // LIB_RCSTRING_LIST_H
