//
//	plasticfs - the plastic file system
//	Copyright (C) 2003 Peter Miller;
//	All rights reserved.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
//
// MANIFEST: interface definition for lib/symtab.cc
//

#ifndef LIB_SYMTAB_H
#define LIB_SYMTAB_H

#pragma interface "libplasticfs::symtab"

#include <rcstring.h>

namespace libplasticfs {


/**
  * The symtab class is used to represent a symbol table.  The indexes are
  * strings, the data may be arbitrary pointers.  Access times are O(1).
  */
class symtab
{
public:
    /**
      * The destructor.
      */
    virtual ~symtab();

    /**
      * The default constructor.
      */
    symtab();

private:
    /**
      * The copy constructor.  Do not use.
      */
    symtab(const symtab &);

    /**
      * The assignment operator.  Do not use.
      */
    symtab &operator=(const symtab &);

    struct row_ty
    {
	row_ty() : data(0), overflow(0) { }

	rcstring key;
	void *data;
	row_ty *overflow;
    };

    void (*reap)(void *);
    row_ty **hash_table;
    typedef rcstring::hash_ty hash_ty;
    hash_ty hash_modulus;
    hash_ty hash_cutover;
    hash_ty hash_cutover_mask;
    hash_ty hash_cutover_split_mask;
    hash_ty hash_split;
    hash_ty hash_load;

public:
    void *query(const rcstring &name) const;
    void assign(const rcstring &name, void *value);
    void remove(const rcstring &name);
    void dump(const char *caption) const;

    struct walk_t
    {
	virtual ~walk_t() {}
	virtual void action(const rcstring &key, void *value) = 0;
    };

    void walk(walk_t &);

    /**
      * The set_reaper method is used to set the reaper function to
      * be used when destroying data.  The default is to do nothing.
      */
    void set_reaper(void(*)(void *));

private:
    void split();
};

} // namespace libplasticfs

#endif // LIB_SYMTAB_H
